@@ -3,11 +3,17 @@
 # Author:: Nathaniel Talbott.
 # Copyright::
 #   * Copyright (c) 2000-2003 Nathaniel Talbott. All rights reserved.
-#   * Copyright (c) 2008-2011 Kouhei Sutou <kou@clear-code.com>
+#   * Copyright (c) 2008-2013 Kouhei Sutou <kou@clear-code.com>
 # License:: Ruby license.
 
+begin
+  require 'io/console'
+rescue LoadError
+end
+
 require 'test/unit/color-scheme'
 require 'test/unit/code-snippet-fetcher'
+require 'test/unit/fault-location-detector'
 require 'test/unit/diff'
 require 'test/unit/ui/testrunner'
 require 'test/unit/ui/testrunnermediator'
@@ -47,6 +53,7 @@ module Test
             @current_output_level = NORMAL
             @faults = []
             @code_snippet_fetcher = CodeSnippetFetcher.new
+            @test_suites = []
           end
 
           private
@@ -87,7 +94,8 @@ module Test
 
           def add_fault(fault)
             @faults << fault
-            output_progress(fault.single_character_display, fault_color(fault))
+            output_progress(fault.single_character_display,
+                            fault_marker_color(fault))
             output_progress_in_detail(fault) if @show_detail_immediately
             @already_outputted = true if fault.critical?
           end
@@ -169,27 +177,43 @@ module Test
           end
 
           def output_fault_in_detail(fault)
-            if fault.is_a?(Failure)
-              output_single(fault.label, fault_color(fault))
-              output(":")
-              output(fault.test_name)
+            if fault.is_a?(Failure) and
+                fault.inspected_expected and fault.inspected_actual
+              output_single("#{fault.label}: ")
+              output(fault.test_name, fault_color(fault))
               output_fault_backtrace(fault)
-              output_fault_message(fault)
+              output_failure_message(fault)
             else
-              output_single(fault.label, fault_color(fault))
               if fault.is_a?(Error)
-                output(": #{fault.test_name}")
-                output(fault.message)
+                output_single("#{fault.label}: ")
+                output_single(fault.test_name, fault_color(fault))
+                output_fault_message(fault)
               else
-                output(": #{fault.message}")
+                output_single(fault.label, fault_color(fault))
+                output_fault_message(fault)
                 output(fault.test_name)
               end
               output_fault_backtrace(fault)
             end
           end
 
+          def output_fault_message(fault)
+            message = fault.message
+            return if message.nil?
+
+            if message.include?("\n")
+              output(":")
+              message.each_line do |line|
+                output("  #{line.chomp}")
+              end
+            else
+              output(": #{message}")
+            end
+          end
+
           def output_fault_backtrace(fault)
             snippet_is_shown = false
+            detector = FaultLocationDetector.new(fault, @code_snippet_fetcher)
             backtrace = fault.location
             # workaround for test-spec. :<
             # see also GitHub:#22
@@ -197,14 +221,14 @@ module Test
             backtrace.each_with_index do |entry, i|
               output(entry)
               next if snippet_is_shown
-              snippet_is_shown = output_code_snippet(entry, fault_color(fault))
+              next unless detector.target?(entry)
+              file, line_number, = detector.split_backtrace_entry(entry)
+              snippet_is_shown = output_code_snippet(file, line_number,
+                                                     fault_color(fault))
             end
           end
 
-          def output_code_snippet(entry, target_line_color=nil)
-            return false unless /\A(.*):(\d+)/ =~ entry
-            file = $1
-            line_number = $2.to_i
+          def output_code_snippet(file, line_number, target_line_color=nil)
             lines = @code_snippet_fetcher.fetch(file, line_number)
             return false if lines.empty?
 
@@ -224,42 +248,50 @@ module Test
             true
           end
 
-          def output_fault_message(fault)
-            if fault.expected.respond_to?(:encoding) and
-                fault.actual.respond_to?(:encoding) and
-                fault.expected.encoding != fault.actual.encoding
+          def output_failure_message(failure)
+            if failure.expected.respond_to?(:encoding) and
+                failure.actual.respond_to?(:encoding) and
+                failure.expected.encoding != failure.actual.encoding
               need_encoding = true
             else
               need_encoding = false
             end
-            output(fault.user_message) if fault.user_message
+            output(failure.user_message) if failure.user_message
             output_single("<")
-            output_single(fault.inspected_expected, color("pass"))
+            output_single(failure.inspected_expected, color("pass"))
             output_single(">")
             if need_encoding
               output_single("(")
-              output_single(fault.expected.encoding.name, color("pass"))
+              output_single(failure.expected.encoding.name, color("pass"))
               output_single(")")
             end
             output(" expected but was")
             output_single("<")
-            output_single(fault.inspected_actual, color("failure"))
+            output_single(failure.inspected_actual, color("failure"))
             output_single(">")
             if need_encoding
               output_single("(")
-              output_single(fault.actual.encoding.name, color("failure"))
+              output_single(failure.actual.encoding.name, color("failure"))
               output_single(")")
             end
             output("")
-            from, to = prepare_for_diff(fault.expected, fault.actual)
+            from, to = prepare_for_diff(failure.expected, failure.actual)
             if from and to
+              if need_encoding
+                unless from.valid_encoding?
+                  from = from.dup.force_encoding("ASCII-8BIT")
+                end
+                unless to.valid_encoding?
+                  to = to.dup.force_encoding("ASCII-8BIT")
+                end
+              end
               from_lines = from.split(/\r?\n/)
               to_lines = to.split(/\r?\n/)
               if need_encoding
                 from_lines << ""
                 to_lines << ""
-                from_lines << "Encoding: #{fault.expected.encoding.name}"
-                to_lines << "Encoding: #{fault.actual.encoding.name}"
+                from_lines << "Encoding: #{failure.expected.encoding.name}"
+                to_lines << "Encoding: #{failure.actual.encoding.name}"
               end
               differ = ColorizedReadableDiffer.new(from_lines, to_lines, self)
               if differ.need_diff?
@@ -282,11 +314,11 @@ module Test
 
           def output_statistics(elapsed_time)
             output("Finished in #{elapsed_time} seconds.")
-            nl
-            output(@result, result_color)
-            output("%g%% passed" % @result.pass_percentage, result_color)
+            output_summary_marker
+            output(@result)
+            output("%g%% passed" % @result.pass_percentage)
             unless elapsed_time.zero?
-              nl
+              output_summary_marker
               test_throughput = @result.run_count / elapsed_time
               assertion_throughput = @result.assertion_count / elapsed_time
               throughput = [
@@ -297,10 +329,20 @@ module Test
             end
           end
 
+          def output_summary_marker
+            term_width = guess_term_width
+            if term_width.zero?
+              marker_width = 6
+            else
+              marker_width = term_width
+            end
+            output("-" * marker_width, summary_marker_color)
+          end
+
           def test_started(test)
             return unless output?(VERBOSE)
 
-            name = test.name.sub(/\(.+?\)\z/, '')
+            name = test.local_name
             right_space = 8 * 2
             left_space = @progress_row_max - right_space
             left_space = left_space - indent.size - name.size
@@ -311,7 +353,7 @@ module Test
 
           def test_finished(test)
             unless @already_outputted
-              output_progress(".", color("pass"))
+              output_progress(".", color("pass-marker"))
             end
             @already_outputted = false
 
@@ -320,7 +362,18 @@ module Test
             output(": (%f)" % (Time.now - @test_start), nil, VERBOSE)
           end
 
+          def suite_name(prefix, suite)
+            name = suite.name
+            if name.nil?
+              "(anonymous)"
+            else
+              name.sub(/\A#{Regexp.escape(prefix)}/, "")
+            end
+          end
+
           def test_suite_started(suite)
+            last_test_suite = @test_suites.last
+            @test_suites << suite
             if @top_level
               @top_level = false
               return
@@ -332,13 +385,15 @@ module Test
             else
               _color = color("case")
             end
-            output_single(suite.name, _color, VERBOSE)
+            prefix = "#{last_test_suite.name}::"
+            output_single(suite_name(prefix, suite), _color, VERBOSE)
             output(": ", nil, VERBOSE)
             @indent += 2
           end
 
           def test_suite_finished(suite)
             @indent -= 2
+            @test_suites.pop
           end
 
           def indent
@@ -384,7 +439,7 @@ module Test
 
           def output_progress_in_detail_marker(fault)
             if @progress_row_max > 0
-              output("=" * @progress_row_max, fault_color(fault))
+              output("=" * @progress_row_max)
             else
               nl
             end
@@ -414,22 +469,29 @@ module Test
             _color
           end
 
+          def fault_class_color_name(fault_class)
+            fault_class.name.split(/::/).last.downcase
+          end
+
           def fault_color(fault)
-            fault_class_color(fault.class)
+            color(fault_class_color_name(fault.class))
           end
 
-          def fault_class_color(fault_class)
-            color(fault_class.name.split(/::/).last.downcase)
+          def fault_marker_color(fault)
+            color("#{fault_class_color_name(fault.class)}-marker")
           end
 
-          def result_color
-            color(@result.status)
+          def summary_marker_color
+            color("#{@result.status}-marker")
           end
 
           def guess_color_availability
             return false unless @output.tty?
+            return true if windows? and ruby_2_0_or_later?
             case ENV["TERM"]
-            when /term(?:-color)?\z/, "screen"
+            when /(?:term|screen)(?:-(?:256)?color)?\z/
+              true
+            when /\Arxvt/
               true
             else
               return true if ENV["EMACS"] == "t"
@@ -437,6 +499,14 @@ module Test
             end
           end
 
+          def windows?
+            /mswin|mingw/ === RUBY_PLATFORM
+          end
+
+          def ruby_2_0_or_later?
+            RUBY_VERSION >= "2.0.0"
+          end
+
           def guess_progress_row_max
             term_width = guess_term_width
             if term_width.zero?
@@ -451,9 +521,30 @@ module Test
           end
 
           def guess_term_width
-            Integer(ENV["COLUMNS"] || ENV["TERM_WIDTH"] || 0)
-          rescue ArgumentError
-            0
+            guess_term_width_from_io || guess_term_width_from_env || 0
+          end
+
+          def guess_term_width_from_io
+            if @output.respond_to?(:winsize)
+              begin
+                @output.winsize[1]
+              rescue SystemCallError
+                nil
+              end
+            else
+              nil
+            end
+          end
+
+          def guess_term_width_from_env
+            env = ENV["COLUMNS"] || ENV["TERM_WIDTH"]
+            return nil if env.nil?
+
+            begin
+              Integer(env)
+            rescue ArgumentError
+              nil
+            end
           end
         end
 
@@ -464,6 +555,7 @@ module Test
           end
 
           def need_diff?(options={})
+            return false if one_line_all_change?
             operations.each do |tag,|
               return true if [:replace, :equal].include?(tag)
             end
@@ -471,16 +563,28 @@ module Test
           end
 
           private
+          def one_line_all_change?
+            return false if operations.size != 1
+
+            tag, from_start, from_end, to_start, to_end = operations.first
+            return false if tag != :replace
+            return false if [from_start, from_end] != [0, 1]
+            return false if [from_start, from_end] != [to_start, to_end]
+
+            _, _, _line_operations = line_operations(@from.first, @to.first)
+            _line_operations.size == 1
+          end
+
           def output_single(something, color=nil)
-            @runner.send(:output_single, something, color)
+            @runner.__send__(:output_single, something, color)
           end
 
           def output(something, color=nil)
-            @runner.send(:output, something, color)
+            @runner.__send__(:output, something, color)
           end
 
           def color(name)
-            @runner.send(:color, name)
+            @runner.__send__(:color, name)
           end
 
           def cut_off_ratio
