@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include "milter-manager.h"
 #include "milter-manager-leader.h"
@@ -136,6 +137,9 @@ static guint  get_max_pending_finished_sessions
 static void   set_max_pending_finished_sessions
                                           (MilterClient *client,
                                            guint         n_sessions);
+static void   workers_created             (MilterClient *client,
+                                           guint         n_workers);
+static void   worker_created              (MilterClient *client);
 
 static void
 milter_manager_class_init (MilterManagerClass *klass)
@@ -189,6 +193,8 @@ milter_manager_class_init (MilterManagerClass *klass)
         get_max_pending_finished_sessions;
     client_class->set_max_pending_finished_sessions =
         set_max_pending_finished_sessions;
+    client_class->workers_created = workers_created;
+    client_class->worker_created = worker_created;
 
     spec = g_param_spec_object("configuration",
                                "Configuration",
@@ -1389,6 +1395,18 @@ set_max_pending_finished_sessions (MilterClient *client, guint n_sessions)
                                                                    n_sessions);
 }
 
+static void
+workers_created (MilterClient *client, guint n_workers)
+{
+    milter_debug("[manager][workers-created] <%d>", n_workers);
+}
+
+static void
+worker_created (MilterClient *client)
+{
+    milter_debug("[manager][worker-created] pid=<%d>", getpid());
+}
+
 MilterManagerConfiguration *
 milter_manager_get_configuration (MilterManager *manager)
 {
