@@ -56,6 +56,7 @@ static MilterManagerTestScenario *main_scenario;
 static gchar *milter_manager_program_name;
 
 static gchar *original_lang;
+static gchar *original_charset;
 
 static gchar *tmp_dir;
 
@@ -177,6 +178,8 @@ cut_setup (void)
 
     original_lang = g_strdup(g_getenv("LANG"));
     g_setenv("LANG", "C", TRUE);
+    original_charset = g_strdup(g_getenv("CHARSET"));
+    g_setenv("CHARSET", "UTF-8", TRUE);
 
     scenario_dir = g_build_filename(milter_test_get_source_dir(),
                                     "test",
@@ -231,6 +234,13 @@ cut_teardown (void)
         g_unsetenv("LANG");
     }
 
+    if (original_charset) {
+        g_setenv("CHARSET", original_charset, TRUE);
+        g_free(original_charset);
+    } else {
+        g_unsetenv("CHARSET");
+    }
+
     if (loop)
         g_object_unref(loop);
 }
@@ -321,7 +331,7 @@ wait_for_manager_ready (const gchar *spec)
                                  &error);
     gcut_assert_error(error);
 
-    timeout_waiting_id = milter_event_loop_add_timeout(loop, 1,
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 3,
                                                        cb_timeout_emitted,
                                                        &timeout_emitted);
     do {
@@ -428,7 +438,11 @@ test_unknown_option (void)
             "Unknown option --nonexistent\n"
             "\n"
             "Usage:\n"
+#if GLIB_CHECK_VERSION(2, 52, 0)
+            "  %s [OPTION…]\n"
+#else
             "  %s [OPTION...]\n"
+#endif
             "\n"
             "Help Options:\n"
             "  -h, --help                                     "
